@@ -1,6 +1,7 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
- * This program is free software; see LICENSE for details. */
+ *
+ * Copyright 1998-2005 Decklin Foster <decklin@red-bean.com>. This
+ * program is free software; please see LICENSE for details. */
 
 #include <X11/Xmd.h>
 #include "aewm.h"
@@ -26,18 +27,20 @@ client_t *find_client(Window w, int mode)
 
 int theight(client_t *c)
 {
+    int font_height;
+
     if (!c) return 0;
 
 #ifdef MWM_HINTS
     if (!c->has_title) return 0;
 #endif
+
 #ifdef XFT
-    return (c->trans ? 0 : xftfont->ascent + xftfont->descent) +
-        2*opt_pad + BW(c);
+    font_height = xftfont->ascent + xftfont->descent;
 #else
-    return (c->trans ? 0 : font->ascent + font->descent) +
-        2*opt_pad + BW(c);
+    font_height = font->ascent + font->descent;
 #endif
+    return c->trans ? font_height / 2 : font_height + 2*opt_pad + BW(c);
 }
 
 /* Attempt to follow the ICCCM by explicity specifying 32 bits for
