@@ -1,6 +1,7 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
- * This program is free software; see LICENSE for details. */
+ *
+ * Copyright 1998-2005 Decklin Foster <decklin@red-bean.com>. This
+ * program is free software; please see LICENSE for details. */
 
 #include <X11/Xatom.h>
 #include "aewm.h"
@@ -8,6 +9,7 @@
 static void handle_button_press(XButtonEvent *);
 static void handle_button_release(XButtonEvent *);
 static void handle_configure_request(XConfigureRequestEvent *);
+static void handle_circulate_request(XCirculateRequestEvent *);
 static void handle_map_request(XMapRequestEvent *);
 static void handle_unmap_event(XUnmapEvent *);
 static void handle_destroy_event(XDestroyWindowEvent *);
@@ -41,6 +43,8 @@ void do_event_loop(void)
                 handle_button_release(&ev.xbutton); break;
             case ConfigureRequest:
                 handle_configure_request(&ev.xconfigurerequest); break;
+            case CirculateRequest:
+                handle_circulate_request(&ev.xcirculaterequest); break;
             case MapRequest:
                 handle_map_request(&ev.xmaprequest); break;
             case UnmapNotify:
@@ -172,6 +176,20 @@ static void handle_configure_request(XConfigureRequestEvent *e)
     XConfigureWindow(dpy, e->window, e->value_mask, &wc);
 }
 
+/* The only window that we will circulate children for is the root
+ * (because nothing else would make sense). After a client requests that
+ * the root's children be circulated, the server will determine which
+ * window needs to be raised or lowered, and so all we have to do is
+ * make it so. */
+
+static void handle_circulate_request(XCirculateRequestEvent *e)
+{
+    if (e->parent == root) {
+        if (e->place == PlaceOnBottom) XLowerWindow(dpy, e->window);
+        else /* e->place == PlaceOnTop */ XRaiseWindow(dpy, e->window);
+    }
+}
+
 /* Two possiblilies if a client is asking to be mapped. One is that
  * it's a new window, so we handle that if it isn't in our clients
  * list anywhere. The other is that it already exists and wants to
