@@ -1,6 +1,7 @@
 /* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
- * This program is free software; see LICENSE for details. */
+ *
+ * Copyright 1998-2005 Decklin Foster <decklin@red-bean.com>. This
+ * program is free software; please see LICENSE for details. */
 
 #include "aewm.h"
 #ifdef GNOME_PDA
@@ -185,30 +186,36 @@ static void init_position(client_t *c)
     }
 
     if (c->size->flags & USPosition) {
+        /* always obey */
         c->x = c->size->x;
         c->y = c->size->y;
-    } else {
-        if (c->size->flags & PPosition) {
+    } else if (c->size->flags & PPosition) {
+        /* only if it's sane. I'm looking at you, Netscape, Emacs, GTK... */
+        if (c->size->x != 0 && c->size->y != 0) {
             c->x = c->size->x;
             c->y = c->size->y;
         }
-        if (c->x < 0) c->x = 0;
-        if (c->y < 0) c->y = 0;
-        if (c->x > xmax) c->x = xmax - theight(c);
-        if (c->y > ymax) c->y = ymax - theight(c);
+    }
 
-        if (c->x == 0 && c->y == 0) {
-            int mouse_x, mouse_y;
-            get_mouse_position(&mouse_x, &mouse_y);
-            if (c->width < xmax)
-                c->x = (mouse_x < xmax ? (mouse_x / (float)xmax) : 1)
-                    * (xmax - c->width - 2*BW(c));
-            if (c->height + theight(c) < ymax)
-                c->y = (mouse_y < ymax ? (mouse_y / (float)ymax) : 1)
-                    * (ymax - c->height - theight(c) - 2*BW(c));
-            c->y += theight(c);
-            gravitate(c, GRAV_UNDO);
-        }
+    /* Even if we got a hint for x and y, we have to check that the
+     * client isn't being insane. If it is, just reset things to 0 so we
+     * can place it ourselves. */
+
+    if (c->x < 0 || c->x > xmax) c->x = 0;
+    if (c->y < 0 || c->y > ymax) c->y = 0;
+
+    /* If it's still at 0,0, we want to place it. */
+    if (c->x == 0 && c->y == 0) {
+        int mouse_x, mouse_y;
+        get_mouse_position(&mouse_x, &mouse_y);
+        if (c->width < xmax)
+            c->x = (mouse_x < xmax ? (mouse_x / (float)xmax) : 1)
+                * (xmax - c->width - 2*BW(c));
+        if (c->height + theight(c) < ymax)
+            c->y = (mouse_y < ymax ? (mouse_y / (float)ymax) : 1)
+                * (ymax - c->height - theight(c) - 2*BW(c));
+        c->y += theight(c);
+        gravitate(c, GRAV_UNDO);
     }
 
 #ifdef GNOME_PDA
